## Naive Bayes Classifier for Sentiment Analysis

import os
import pandas as pd
import matplotlib.pyplot as plt

# Global class labels.
POS_LABEL = 'POS_class'
NEG_LABEL = 'NEG_class'

class NaiveBayes:
    """A Naive Bayes model for text classification."""

    def __init__(self):
        # Vocabulary is a set that stores every word seen in the training data
        self.vocab = set()

        # class_total_doc_counts is a dictionary that maps a class (i.e., pos/neg) to
        # the number of documents in the trainning set of that class
        self.class_total_doc_counts = {POS_LABEL: 0.0,
                                       NEG_LABEL: 0.0}

        # class_total_word_counts is a dictionary that maps a class (i.e., pos/neg) to
        # the number of words in the training set in documents of that class
        self.class_total_word_counts = {POS_LABEL: 0.0,
                                        NEG_LABEL: 0.0}

        # class_word_counts is a dictionary of dictionaries. It maps a class (i.e.,
        # pos/neg) to a dictionary of word counts. For example:
        #    self.class_word_counts[POS_LABEL]['awesome']
        # stores the number of times the word 'awesome' appears in documents
        # of the positive class in the training documents.
        self.class_word_counts = {POS_LABEL: {}, NEG_LABEL: {}}

    def tokenize_doc(self, doc):
        """
        IMPLEMENT ME!

        Tokenize a document and return its bag-of-words representation.
        doc - a string representing a document.
        returns a dictionary mapping each word to the number of times it appears in doc.
        """
        bag_of_word = {}
        doc = doc.split()
        for word in doc:
            if word in bag_of_word:
                bag_of_word[word] += 1
            else:
                bag_of_word[word] = 1
        return bag_of_word

    def remove_stop_words(self, bag_of_words):
        stopwords = []
        with open('./stopwords.txt', 'r', encoding='utf-8') as f:
            pass

    def train(self, train_data, num_docs=None):
        if num_docs is not None:
            print('training on {} documents'.format(num_docs))
        print("\33[92mtraining model\33[0m")
        for content, label in train_data:
            self.tokenize_and_update_model(content, label)
        self.report_statistics_after_training()

    def report_statistics_after_training(self):
        """
        Report a number of statistics after training.
        """

        print("REPORTING CORPUS STATISTICS")

        print("NUMBER OF DOCUMENTS IN POSITIVE CLASS:", self.class_total_doc_counts[POS_LABEL])
        print("NUMBER OF DOCUMENTS IN NEGATIVE CLASS:", self.class_total_doc_counts[NEG_LABEL])
        print("NUMBER OF WORDS IN POSITIVE CLASS:", self.class_total_word_counts[POS_LABEL])
        print("NUMBER OF WORDS IN NEGATIVE CLASS:", self.class_total_word_counts[NEG_LABEL])
        print("VOCABULARY SIZE: NUMBER OF UNIQUE WORDTYPES IN TRAINING CORPUS:", len(self.vocab))

    def update_model(self, bag_of_words, label):
        """
        IMPLEMENT ME!

        Update internal statistics given a document represented as a bag-of-words
        tokenized_document - a map from words to their counts
        label - the class of the document whose bag-of-words representation was input
        This function doesn't return anything but should update a number of internal
        statistics. Specifically, it updates:
          - the internal map the counts, per class, how many times each word was
            seen (self.class_word_counts)
          - the number of words seen for each class (self.class_total_word_counts)
          - the vocabulary seen so far (self.vocab)
          - the number of documents seen of each class (self.class_total_doc_counts)
        """

        print("\33[92mupdating model\33[0m")        
        for word in bag_of_words:
            if word not in self.vocab:
                self.vocab.add(word)
            self.class_word_counts[label][word] += bag_of_words[word]
            self.class_total_word_counts[label] += bag_of_words[word]
        self.class_total_doc_counts[label] += 1

    def tokenize_and_update_model(self, doc, label):
        """
        Tokenizes a document doc and updates internal count statistics.
        doc - a string representing a document.
        label - the sentiment of the document (either postive or negative)
        stop_word - a boolean flag indicating whether to stop word or not

        Make sure when tokenizing to lower case all of the tokens!
        """

        bow = tokenize_doc(doc)
        self.update_model(bow, label)

    def top_n_words(self, label, n):
        """
        Implement me!

        Returns the most frequent n tokens for documents with class 'label'.
        """
        pass

    def p_word_given_label(self, word, label):
        """
        Implement me!

        Returns the probability of word given label (i.e., P(word|label))
        according to this NB model.
        """
        pass


    def p_word_given_label_and_psuedocount(self, word, label, alpha=1.0):
        """
        Implement me!

        Returns the probability of word given label wrt psuedo counts.
        alpha - psuedocount parameter
        """
        pass
    
    def log_likelihood(self, bag_of_word, label, alpha=1.0):
        """
        Computes the log likelihood of a set of words give a label and psuedocount.
        bow - a bag of words (i.e., a tokenized document)
        label - either the positive or negative label
        alpha - float; psuedocount parameter
        """
        pass

    def log_prior(self, label):
        """
        Implement me!

        Returns a float representing the fraction of training documents
        that are of class 'label'.
        """
        pass

    def unnormalized_log_posterior(self, bag_of_word, label, alpha=1.0):
        """
        Implement me!

        alpha - psuedocount parameter
        bow - a bag of words (i.e., a tokenized document)
        Computes the unnormalized log posterior (of doc being of class 'label').
        """
        pass

    def classify(self, bag_of_words, alpha=1.0):
        """
        Implement me!

        alpha - psuedocount parameter.
        bow - a bag of words (i.e., a tokenized document)

        Compares the unnormalized log posterior for doc for both the positive
        and negative classes and returns the either POS_LABEL or NEG_LABEL
        (depending on which resulted in the higher unnormalized log posterior).
        """
        pass

    def likelihood_ratio(self, word, alpha=1.0):
        """
        Implement me!

        alpha - psuedocount parameter.
        Returns the ratio of P(word|pos) to P(word|neg).
        """
        pass

    def predict(self, test_data, alpha=1.0):
        """
        
        """
        pass
    
    def evaluate_classifier_accuracy(self, test_data, test_labels, alpha=1.0):
        """
        Implement me!

        alpha - psuedocount parameter.
        This function should go through the test data, classify each instance and
        compute the accuracy of the classifier (the fraction of classifications
        the classifier gets right.
        """
        pass

def tokenize_doc(doc):
    bag_of_word = {}
    doc = doc.split()
    for word in doc:
        if word in bag_of_word:
            bag_of_word[word] += 1
        else:
            bag_of_word[word] = 1
    return bag_of_word

def plot_top_word_freq(model, label, num_of_words, title, color, ax):
    if ax is None:
        plt.gca() # get current axis
    else:
        plt.sca(ax) # set current axis
    top_words = model.top_n_words(label, num_of_words)
    top_words = [word[0] for word in top_words]
    top_words_freq = [model.class_word_counts[label][word] for word in top_words]
    plt.bar(top_words, top_words_freq, color=color)
    plt.xticks(range(len(top_words)), top_words, rotation=90)
    plt.title(f"Top {num_of_words} words in {title}")
    plt.xlabel("Words")
    plt.ylabel("Frequency")

def plot_psuedocount_vs_accuracy(psuedocounts, accuracies):
    """
    A function to plot psuedocounts vs. accuries. You may want to modify this function
    to enhance your plot.
    """

    plt.plot(psuedocounts, accuracies)
    plt.xlabel('Psuedocount Parameter')
    plt.ylabel('Accuracy (%)')
    plt.title('Psuedocount Parameter vs. Accuracy Experiment')
    plt.show()


def produce_results():
    # PRELIMINARIES
    # uncomment the following 9 lines after you've implemented tokenize_doc
    sentence = "this sample doc has   words that  repeat repeat and repeat"
    tokenized_doc = tokenize_doc(sentence)
    # print("TOKENIZED DOC:", tokenized_doc)

    assertion_dict = {'this': 1, 'sample': 1, 'doc': 1, 'has': 1, 'words': 1, 'that': 1, 'repeat': 3, 'and': 1}
    pass_flag = True
    for word in assertion_dict:
        if assertion_dict[word] != tokenized_doc[word]:
            pass_flag = False
    print("TEST tokenize_doc: PASSED" if pass_flag else "Test tokenize_doc: FAILED")

    # assert tokenized_doc['this'] == 1
    # assert tokenized_doc['sample'] == 1
    # assert tokenized_doc['doc'] == 1
    # assert tokenized_doc['has'] == 1
    # assert tokenized_doc['words'] == 1
    # assert tokenized_doc['that'] == 1
    # assert tokenized_doc['repeat'] == 3
    # assert tokenized_doc['and'] == 1

    # QUESTION 1.1
    # Implementation only
    # assert update_model(tokenized_doc, POS_LABEL) is doing what it should
    sentence = "this sample doc has   words that  repeat repeat and repeat"
    tokenized_doc = tokenize_doc(sentence)
    nb = NaiveBayes()
    nb.update_model(tokenized_doc, POS_LABEL)
    print()
    print(nb.class_total_doc_counts[POS_LABEL])
    assertion_dict = {'this': 1, 'sample': 1, 'doc': 1, 'has': 1, 'words': 1, 'that': 1, 'repeat': 3, 'and': 1}
    pass_flag = True
    for word in assertion_dict:
        if assertion_dict[word] != nb.class_word_counts[POS_LABEL][word]:
            pass_flag = False
    print("TEST update_model: PASSED" if pass_flag else "Test update_model: FAILED")

if __name__ == '__main__':
    # nb = NaiveBayes()
    # nb.train_model()
    # nb.train_model(num_docs=20)
    produce_results()